@@ -8,13 +8,12 @@ import json
 from io import StringIO
 from os import PathLike
 from pathlib import Path
-from typing import Union, Optional, Dict
+from typing import Union, Optional, Dict, Any, Callable
 
 from box.exceptions import BoxError
 
 pyyaml_available = True
 ruamel_available = True
-toml_available = True
 msgpack_available = True
 
 try:
@@ -30,11 +29,72 @@ try:
 except ImportError:
     pyyaml_available = False
 
+toml_read_library: Optional[Any] = None
+toml_write_library: Optional[Any] = None
+toml_decode_error: Optional[Callable] = None
+
+__all__ = [
+    "_to_json",
+    "_to_yaml",
+    "_to_toml",
+    "_to_csv",
+    "_to_msgpack",
+    "_from_json",
+    "_from_yaml",
+    "_from_toml",
+    "_from_csv",
+    "_from_msgpack",
+]
+
+
+class BoxTomlDecodeError(BoxError):
+    """Toml Decode Error"""
+
+
 try:
     import toml
 except ImportError:
-    toml = None  # type: ignore
-    toml_available = False
+    pass
+else:
+    toml_read_library = toml
+    toml_write_library = toml
+    toml_decode_error = toml.TomlDecodeError
+
+    class BoxTomlDecodeError(BoxError, toml.TomlDecodeError):  # type: ignore
+        """Toml Decode Error"""
+
+
+try:
+    import tomllib
+except ImportError:
+    pass
+else:
+    toml_read_library = tomllib
+    toml_decode_error = tomllib.TOMLDecodeError
+
+    class BoxTomlDecodeError(BoxError, tomllib.TOMLDecodeError):  # type: ignore
+        """Toml Decode Error"""
+
+
+try:
+    import tomli
+except ImportError:
+    pass
+else:
+    toml_read_library = tomli
+    toml_decode_error = tomli.TOMLDecodeError
+
+    class BoxTomlDecodeError(BoxError, tomli.TOMLDecodeError):  # type: ignore
+        """Toml Decode Error"""
+
+
+try:
+    import tomli_w
+except ImportError:
+    pass
+else:
+    toml_write_library = tomli_w
+
 
 try:
     import msgpack  # type: ignore
@@ -77,7 +137,7 @@ def _exists(filename: Union[str, PathLike], create: bool = False) -> Path:
 
 
 def _to_json(
-    obj, filename: Union[str, PathLike] = None, encoding: str = "utf-8", errors: str = "strict", **json_kwargs
+    obj, filename: Optional[Union[str, PathLike]] = None, encoding: str = "utf-8", errors: str = "strict", **json_kwargs
 ):
     if filename:
         _exists(filename, create=True)
@@ -88,8 +148,8 @@ def _to_json(
 
 
 def _from_json(
-    json_string: str = None,
-    filename: Union[str, PathLike] = None,
+    json_string: Optional[str] = None,
+    filename: Optional[Union[str, PathLike]] = None,
     encoding: str = "utf-8",
     errors: str = "strict",
     multiline: bool = False,
@@ -114,7 +174,7 @@ def _from_json(
 
 def _to_yaml(
     obj,
-    filename: Union[str, PathLike] = None,
+    filename: Optional[Union[str, PathLike]] = None,
     default_flow_style: bool = False,
     encoding: str = "utf-8",
     errors: str = "strict",
@@ -154,8 +214,8 @@ def _to_yaml(
 
 
 def _from_yaml(
-    yaml_string: str = None,
-    filename: Union[str, PathLike] = None,
+    yaml_string: Optional[str] = None,
+    filename: Optional[Union[str, PathLike]] = None,
     encoding: str = "utf-8",
     errors: str = "strict",
     ruamel_typ: str = "rt",
@@ -195,30 +255,50 @@ def _from_yaml(
     return data
 
 
-def _to_toml(obj, filename: Union[str, PathLike] = None, encoding: str = "utf-8", errors: str = "strict"):
+def _to_toml(obj, filename: Optional[Union[str, PathLike]] = None, encoding: str = "utf-8", errors: str = "strict"):
     if filename:
         _exists(filename, create=True)
-        with open(filename, "w", encoding=encoding, errors=errors) as f:
-            toml.dump(obj, f)
+        if toml_write_library.__name__ == "toml":  # type: ignore
+            with open(filename, "w", encoding=encoding, errors=errors) as f:
+                try:
+                    toml_write_library.dump(obj, f)  # type: ignore
+                except toml_decode_error as err:  # type: ignore
+                    raise BoxTomlDecodeError(err) from err
+        else:
+            with open(filename, "wb") as f:
+                try:
+                    toml_write_library.dump(obj, f)  # type: ignore
+                except toml_decode_error as err:  # type: ignore
+                    raise BoxTomlDecodeError(err) from err
     else:
-        return toml.dumps(obj)
+        try:
+            return toml_write_library.dumps(obj)  # type: ignore
+        except toml_decode_error as err:  # type: ignore
+            raise BoxTomlDecodeError(err) from err
 
 
 def _from_toml(
-    toml_string: str = None, filename: Union[str, PathLike] = None, encoding: str = "utf-8", errors: str = "strict"
+    toml_string: Optional[str] = None,
+    filename: Optional[Union[str, PathLike]] = None,
+    encoding: str = "utf-8",
+    errors: str = "strict",
 ):
     if filename:
         _exists(filename)
-        with open(filename, "r", encoding=encoding, errors=errors) as f:
-            data = toml.load(f)
+        if toml_read_library.__name__ == "toml":  # type: ignore
+            with open(filename, "r", encoding=encoding, errors=errors) as f:
+                data = toml_read_library.load(f)  # type: ignore
+        else:
+            with open(filename, "rb") as f:
+                data = toml_read_library.load(f)  # type: ignore
     elif toml_string:
-        data = toml.loads(toml_string)
+        data = toml_read_library.loads(toml_string)  # type: ignore
     else:
         raise BoxError("from_toml requires a string or filename")
     return data
 
 
-def _to_msgpack(obj, filename: Union[str, PathLike] = None, **kwargs):
+def _to_msgpack(obj, filename: Optional[Union[str, PathLike]] = None, **kwargs):
     if filename:
         _exists(filename, create=True)
         with open(filename, "wb") as f:
@@ -227,7 +307,7 @@ def _to_msgpack(obj, filename: Union[str, PathLike] = None, **kwargs):
         return msgpack.packb(obj, **kwargs)
 
 
-def _from_msgpack(msgpack_bytes: bytes = None, filename: Union[str, PathLike] = None, **kwargs):
+def _from_msgpack(msgpack_bytes: Optional[bytes] = None, filename: Optional[Union[str, PathLike]] = None, **kwargs):
     if filename:
         _exists(filename)
         with open(filename, "rb") as f:
@@ -239,7 +319,9 @@ def _from_msgpack(msgpack_bytes: bytes = None, filename: Union[str, PathLike] =
     return data
 
 
-def _to_csv(box_list, filename: Union[str, PathLike] = None, encoding: str = "utf-8", errors: str = "strict", **kwargs):
+def _to_csv(
+    box_list, filename: Optional[Union[str, PathLike]] = None, encoding: str = "utf-8", errors: str = "strict", **kwargs
+):
     csv_column_names = list(box_list[0].keys())
     for row in box_list:
         if list(row.keys()) != csv_column_names:
@@ -260,8 +342,8 @@ def _to_csv(box_list, filename: Union[str, PathLike] = None, encoding: str = "ut
 
 
 def _from_csv(
-    csv_string: str = None,
-    filename: Union[str, PathLike] = None,
+    csv_string: Optional[str] = None,
+    filename: Optional[Union[str, PathLike]] = None,
     encoding: str = "utf-8",
     errors: str = "strict",
     **kwargs,
